=== modified file 'lib/lp/registry/stories/webservice/xx-derivedistroseries.txt'
--- lib/lp/registry/stories/webservice/xx-derivedistroseries.txt	2011-06-09 10:50:25 +0000
+++ lib/lp/registry/stories/webservice/xx-derivedistroseries.txt	2011-10-07 16:05:13 +0000
@@ -11,11 +11,14 @@
     >>> from lp.testing.sampledata import ADMIN_EMAIL
     >>> from canonical.launchpad.testing.pages import webservice_for_person
     >>> from canonical.launchpad.webapp.interfaces import OAuthPermission
+    >>> from zope.security.proxy import removeSecurityProxy
 
     >>> login(ADMIN_EMAIL)
 
     >>> soyuz_team = factory.makeTeam(name='soyuz-team')
     >>> parent_series = factory.makeDistroSeries(name="parentseries")
+    >>> arch = factory.makeDistroArchSeries(distroseries=parent_series)
+    >>> removeSecurityProxy(parent_series).nominatedarchindep = arch
     >>> child_series = factory.makeDistroSeries(name='child1')
     >>> child_series_with_parent = factory.makeDistroSeries(
     ...     name='child-with-parent')

=== modified file 'lib/lp/registry/tests/test_initderiveddistroseries.py'
--- lib/lp/registry/tests/test_initderiveddistroseries.py	2011-08-22 11:39:17 +0000
+++ lib/lp/registry/tests/test_initderiveddistroseries.py	2011-10-07 16:05:13 +0000
@@ -42,6 +42,8 @@
     def setUp(self):
         super(TestDeriveDistroSeries, self).setUp()
         self.parent = self.factory.makeDistroSeries()
+        arch = self.factory.makeDistroArchSeries(distroseries=self.parent)
+        removeSecurityProxy(self.parent).nominatedarchindep = arch
         self.child = self.factory.makeDistroSeries()
         removeSecurityProxy(self.child).driver = self.factory.makePerson()
         login_person(self.child.driver)
@@ -187,11 +189,11 @@
         res = self.create2archParentAndSource(packages={'p1': '1.1'})
         parent, parent_das, parent_das2, source = res
         # Create builds for the architecture of parent_das2.
-        source.createMissingBuilds(architectures_available=[parent_das2])
+        source.createMissingBuilds(architectures_available=[parent_das])
         child = self.factory.makeDistroSeries(
             distribution=parent.distribution, previous_series=parent)
 
-        # Initialize only with parent_das's architecture. The build is
+        # Initialize only with parent_das2's architecture. The build is
         # in the other architecture so no exception should be raised.
         child.initDerivedDistroSeries(
-            child.driver, [parent.id], (parent_das.architecturetag, ), ())
+            child.driver, [parent.id], (parent_das2.architecturetag, ), ())

=== modified file 'lib/lp/soyuz/scripts/initialize_distroseries.py'
--- lib/lp/soyuz/scripts/initialize_distroseries.py	2011-10-03 09:28:45 +0000
+++ lib/lp/soyuz/scripts/initialize_distroseries.py	2011-10-07 09:30:44 +0000
@@ -157,11 +157,22 @@
                  ".").format(
                     child=self.distroseries))
         self._checkParents()
+        self._checkArchindep()
         for parent in self.derivation_parents:
             self._checkBuilds(parent)
             self._checkQueue(parent)
         self._checkSeries()
 
+    def _checkArchindep(self):
+        # Check that the child distroseries has an architecture to
+        # build architecture independent binaries.
+        potential_nominated_arches = self._potential_nominated_arches(
+             self.derivation_parents)
+        if len(potential_nominated_arches) == 0:
+            raise InitializationError(
+                "The distroseries has no architectures selected to "
+                 "build architecture independent binaries.")
+
     def _checkPublisherConfig(self):
         """A series cannot be initialized if it has no publisher config
         set up.
@@ -365,9 +376,23 @@
             """ % (sqlvalues(self.distroseries, self.distroseries.owner)
             + (das_filter, )))
         self._store.flush()
-        # Take nominatedarchindep from the first parent.
-        self.distroseries.nominatedarchindep = self.distroseries[
-            self.derivation_parents[0].nominatedarchindep.architecturetag]
+        # Select the arch-indep builder from the intersection between
+        # the selected architectures and the list of the parent's
+        # arch-indep builders.
+        arch_tag = self._potential_nominated_arches(
+            self.derivation_parents).pop()
+        self.distroseries.nominatedarchindep = self.distroseries[arch_tag]
+
+    def _potential_nominated_arches(self, parent_list):
+        parent_indep_archtags = set(
+            parent.nominatedarchindep.architecturetag
+            for parent in parent_list
+            if parent.nominatedarchindep is not None)
+
+        if len(self.arches) == 0:
+            return parent_indep_archtags
+        else:
+            return parent_indep_archtags.intersection(self.arches)
 
     def _copy_packages(self):
         # Perform the copies

=== modified file 'lib/lp/soyuz/scripts/tests/test_initialize_distroseries.py'
--- lib/lp/soyuz/scripts/tests/test_initialize_distroseries.py	2011-10-03 09:28:45 +0000
+++ lib/lp/soyuz/scripts/tests/test_initialize_distroseries.py	2011-10-07 07:57:12 +0000
@@ -259,7 +259,7 @@
         parent, parent_das, parent_das2, source = res
         # Create builds for the architecture of parent_das2.
         source.createMissingBuilds(architectures_available=[parent_das2])
-        # Initialize only with parent_das2's architecture.
+        # Initialize only with parent_das's architecture.
         child = self.factory.makeDistroSeries()
         ids = InitializeDistroSeries(
             child, [parent.id], arches=[parent_das2.architecturetag])
@@ -293,13 +293,13 @@
         # architecture we're initializing with, IDS will succeed.
         res = self.create2archParentAndSource(packages={'p1': '1.1'})
         parent, parent_das, parent_das2, source = res
-        # Create builds for the architecture of parent_das2.
-        source.createMissingBuilds(architectures_available=[parent_das2])
-        # Initialize only with parent_das's architecture.
+        # Create builds for the architecture of parent_das.
+        source.createMissingBuilds(architectures_available=[parent_das])
+        # Initialize only with parent_das2's architecture.
         child = self.factory.makeDistroSeries(
             distribution=parent.distribution, previous_series=parent)
         ids = InitializeDistroSeries(
-            child, arches=[parent_das.architecturetag])
+            child, arches=[parent_das2.architecturetag])
 
         # No error is raised because we're initializing only the architecture
         # which has no pending builds in it.
@@ -1383,3 +1383,39 @@
         self.assertEqual([], self.getWaitingJobs(child, 'p2', prev_parent2))
         self.assertNotEqual([], self.getWaitingJobs(child, 'p1', parent3))
         self.assertEqual([], self.getWaitingJobs(child, 'p3', parent3))
+
+    def test_multiple_parents_child_nominatedarchindep(self):
+        # If the list of the selected architectures and the list of the
+        # nominatedarchindep for all the parent intersect, the child's
+        # nominatedarchindep is taken from the intersection of the two
+        # lists.
+        parent1, unused = self.setupParent(
+            packages={}, arch_tag='i386')
+        parent2, unused = self.setupParent(
+            packages={}, arch_tag='amd64')
+        child = self._fullInitialize(
+            [parent1, parent2],
+            arches=[parent2.nominatedarchindep.architecturetag])
+        self.assertEqual(
+            child.nominatedarchindep.architecturetag,
+            parent2.nominatedarchindep.architecturetag)
+
+    def test_multiple_parents_no_child_nominatedarchindep(self):
+        # If the list of the selected architectures and the list of the
+        # nominatedarchindep for all the parents don't intersect, an
+        # error is raised because it means that the child won't have an
+        # architecture to build architecture independent binaries.
+        parent1, unused = self.setupParent(
+            packages={}, arch_tag='i386')
+        self.setupDas(parent1, 'hppa', 'powerpc')
+        parent2, unused = self.setupParent(
+            packages={}, arch_tag='amd64')
+        child = self.factory.makeDistroSeries()
+        ids = InitializeDistroSeries(
+            child, [parent1.id, parent2.id],
+            arches=['powerpc'])
+        self.assertRaisesWithContent(
+            InitializationError,
+            ("The distroseries has no architectures selected to "
+             "build architecture independent binaries."),
+            ids.check)

